@@ -69,12 +69,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <X11/Xmd.h> 
 #endif
 extern "C" {
-#ifdef Q_OS_UNIX
 #include <KDE-ICE/ICElib.h>
 #include <KDE-ICE/ICEutil.h>
 #include <KDE-ICE/ICEmsg.h>
 #include <KDE-ICE/ICEproto.h>
-#endif
 }
 
 // #define DCOPCLIENT_DEBUG	1
@@ -116,16 +114,12 @@ void unregisterLocalClient( const QCString &_appId )
 
 template class QPtrList<DCOPObjectProxy>;
 template class QPtrList<DCOPClientTransaction>;
-#ifdef Q_OS_UNIX
 template class QPtrList<_IceConn>;
-#endif
 
 struct DCOPClientMessage
 {
     int opcode;
-#ifdef Q_OS_UNIX
     CARD32 key;
-#endif
     QByteArray data;
 };
 
@@ -156,9 +150,7 @@ class DCOPClientPrivate
 public:
     DCOPClient *parent;
     QCString appId;
-#ifdef Q_OS_UNIX
     IceConn iceConn;
-#endif
     int majorOpcode; // major opcode negotiated w/server and used to tag all comms.
 
     int majorVersion, minorVersion; // protocol versions negotiated w/server
@@ -182,7 +174,6 @@ public:
     Q_INT32 transactionId;
     int opcode;
 
-#ifdef Q_OS_UNIX
     // Special key values:
     // 0 : Not specified
     // 1 : DCOPSend
@@ -191,7 +182,6 @@ public:
     CARD32 key;
     CARD32 currentKey; 
     CARD32 currentKeySaved;
-#endif
 
     QTimer postMessageTimer;
     QPtrList<DCOPClientMessage> messages;
@@ -214,14 +204,20 @@ class DCOPClientTransaction
 {
 public:
     Q_INT32 id;
-#ifdef Q_OS_UNIX
     CARD32 key;
-#endif
     QCString senderId;
 };
 
 QCString DCOPClient::iceauthPath()
 {
+#ifdef Q_OS_WIN32
+	char	szPath[512];
+	char *	pszFilePart;
+	int		ret;
+	ret = SearchPathA(NULL,"iceauth.exe",NULL,sizeof(szPath)/sizeof(szPath[0]),szPath,&pszFilePart);
+	if(ret != 0)
+		return QCString(szPath);
+#else
     QCString path = ::getenv("PATH");
     if (path.isEmpty())
         path = "/bin:/usr/bin";
@@ -237,6 +233,7 @@ QCString DCOPClient::iceauthPath()
    
         fPath = strtok(NULL, ":\b");
     }
+#endif
     return 0;
 }
 
@@ -245,8 +242,9 @@ static QCString dcopServerFile(const QCString &hostname, bool old)
     QCString fName = ::getenv("DCOPAUTHORITY");
     if (!old && !fName.isEmpty())
         return fName;
-    
-    fName = ::getenv("HOME");
+
+    fName = QFile::encodeName( QDir::homeDirPath() );
+//    fName = ::getenv("HOME");
     if (fName.isEmpty())
     {
         fprintf(stderr, "Aborting. $HOME is not set.\n");
@@ -312,9 +310,7 @@ QCString DCOPClient::dcopServerFileOld(const QCString &hostname)
 
 const char* DCOPClientPrivate::serverAddr = 0;
 
-#ifdef Q_OS_UNIX
 static void DCOPProcessInternal( DCOPClientPrivate *d, int opcode, CARD32 key, const QByteArray& dataReceived, bool canPost  );
-#endif
 
 void DCOPClient::handleAsyncReply(ReplyStruct *replyStruct)
 {
@@ -329,7 +325,6 @@ void DCOPClient::handleAsyncReply(ReplyStruct *replyStruct)
     delete replyStruct;
 }
 
-#ifdef Q_OS_UNIX
 /**
  * Callback for ICE.
  */
@@ -435,11 +430,9 @@ static void DCOPProcessMessage(IceConn iceConn, IcePointer clientObject,
         DCOPProcessInternal( d, opcode, key, dataReceived, true );
     }
 }
-#endif
 
 void DCOPClient::processPostedMessagesInternal()
 {
-#ifdef Q_OS_UNIX
     if ( d->messages.isEmpty() )
         return;
     QPtrListIterator<DCOPClientMessage> it (d->messages );
@@ -455,10 +448,8 @@ void DCOPClient::processPostedMessagesInternal()
     }
     if ( !d->messages.isEmpty() )
         d->postMessageTimer.start( 100, true );
-#endif
 }
 
-#ifdef Q_OS_UNIX
 /**
    Processes DCOPCall, DCOPFind and DCOPSend
  */
@@ -583,7 +574,6 @@ void DCOPProcessInternal( DCOPClientPrivate *d, int opcode, CARD32 key, const QB
 static IcePoVersionRec DCOPClientVersions[] = {
     { DCOPVersionMajor, DCOPVersionMinor,  DCOPProcessMessage }
 };
-#endif
 
 
 static DCOPClient* dcop_main_client = 0;
@@ -603,11 +593,9 @@ DCOPClient::DCOPClient()
 {
     d = new DCOPClientPrivate;
     d->parent = this;
-#ifdef Q_OS_UNIX
     d->iceConn = 0L;
     d->key = 0;
     d->currentKey = 0;
-#endif
     d->majorOpcode = 0;
     d->appId = 0;
     d->notifier = 0L;
@@ -646,11 +634,9 @@ DCOPClient::~DCOPClient()
         delete msg;
     }
 #endif
-#ifdef Q_OS_UNIX
     if (d->iceConn)
         if (IceConnectionStatus(d->iceConn) == IceConnectAccepted)
             detach();
-#endif
 
     if (d->registered)
         unregisterLocalClient( d->appId );
@@ -696,12 +682,20 @@ void DCOPClient::bindToApp()
 
 void DCOPClient::suspend()
 {
+#ifdef Q_WS_WIN //TODO: remove (win32 ports sometimes do not create notifiers)
+    if (!d->notifier)
+        return;
+#endif
     assert(d->notifier); // Suspending makes no sense if we didn't had a qApp yet
     d->notifier->setEnabled(false);
 }
 
 void DCOPClient::resume()
 {
+#ifdef Q_WS_WIN //TODO: remove
+    if (!d->notifier)
+        return;
+#endif
     assert(d->notifier); // Should never happen
     d->notifier->setEnabled(true);
 }
@@ -729,6 +723,12 @@ static bool peerIsUs(int sockfd)
 // Check whether the socket is owned by the same user.
 static bool isServerSocketOwnedByUser(const char*server)
 {
+#ifdef Q_OS_WIN
+    if (strncmp(server, "tcp/", 4) != 0)
+        return false; // Not a local socket -> foreign.
+	else
+		return true;
+#else
     if (strncmp(server, "local/", 6) != 0)
         return false; // Not a local socket -> foreign.
     const char *path = strchr(server, KPATH_SEPARATOR);
@@ -741,13 +741,13 @@ static bool isServerSocketOwnedByUser(const char*server)
         return false;
 
     return (stat_buf.st_uid == getuid());
+#endif
 }
 #endif
 
 
 bool DCOPClient::attachInternal( bool registerAsAnonymous )
 {
-#ifdef Q_OS_UNIX
     char errBuf[1024];
 
     if ( isAttached() )
@@ -769,20 +769,20 @@ bool DCOPClient::attachInternal( bool registerAsAnonymous )
     if (!d->serverAddr) {
         // here, we obtain the list of possible DCOP connections,
         // and attach to them.
-        QString dcopSrv;
+        QCString dcopSrv;
         dcopSrv = ::getenv("DCOPSERVER");
         if (dcopSrv.isEmpty()) {
-            QString fName = dcopServerFile();
-            QFile f(fName);
+            QCString fName = dcopServerFile();
+            QFile f(QFile::decodeName(fName));
             if (!f.open(IO_ReadOnly)) {
-                emit attachFailed(QString::fromLatin1( "Could not read network connection list.\n" )+fName);
+                emit attachFailed(QString::fromLatin1( "Could not read network connection list.\n" )+QFile::decodeName(fName));
                 return false;
             }
             int size = QMIN( 1024, f.size() ); // protection against a huge file
             QCString contents( size+1 );
             if ( f.readBlock( contents.data(), size ) != size )
             {
-               qDebug("Error reading from %s, didn't read the expected %d bytes", fName.latin1(), size);
+               qDebug("Error reading from %s, didn't read the expected %d bytes", fName.data(), size);
                // Should we abort ?
             }
             contents[size] = '\0';
@@ -790,24 +790,26 @@ bool DCOPClient::attachInternal( bool registerAsAnonymous )
             if ( pos == -1 ) // Shouldn't happen
             {
                 qDebug("Only one line in dcopserver file !: %s", contents.data());
-                dcopSrv = QString::fromLatin1(contents);
+                dcopSrv = contents;
             }
             else
             {
-                dcopSrv = QString::fromLatin1(contents.left( pos ));
+				if(contents[pos - 1] == '\r')	// check for windows end of line
+					pos--;
+                dcopSrv = contents.left( pos );
 //#ifndef NDEBUG
-//                qDebug("dcopserver address: %s", dcopSrv.latin1());
+//                qDebug("dcopserver address: %s", dcopSrv.data());
 //#endif
             }
         }
-        d->serverAddr = qstrdup( const_cast<char *>(dcopSrv.latin1()) );
+        d->serverAddr = qstrdup( const_cast<char *>(dcopSrv.data()) );
         bClearServerAddr = true;
     }
 
     if ((d->iceConn = IceOpenConnection(const_cast<char*>(d->serverAddr),
                                         static_cast<IcePointer>(this), False, d->majorOpcode,
                                         sizeof(errBuf), errBuf)) == 0L) {
-        qDebug("DCOPClient::attachInternal. Attach failed %s", errBuf ? errBuf : "");
+        qDebug("DCOPClient::attachInternal. Attach failed %s", errBuf);
         d->iceConn = 0;
         if (bClearServerAddr) {
            delete [] d->serverAddr;
@@ -874,15 +876,11 @@ bool DCOPClient::attachInternal( bool registerAsAnonymous )
         registerAs( "anonymous", true );
 
     return true;
-#else //!Q_OS_UNIX
-    return false;
-#endif
 }
 
 
 bool DCOPClient::detach()
 {
-#ifdef Q_OS_UNIX
     int status;
 
     if (d->iceConn) {
@@ -902,21 +900,14 @@ bool DCOPClient::detach()
     d->registered = false;
     d->foreign_server = true;
     return true;
-#else //!Q_OS_UNIX
-    return false;
-#endif
 }
 
 bool DCOPClient::isAttached() const
 {
-#ifdef Q_OS_UNIX
     if (!d->iceConn)
         return false;
 
     return (IceConnectionStatus(d->iceConn) == IceConnectAccepted);
-#else //!Q_OS_UNIX
-    return false;
-#endif
 }
 
 bool DCOPClient::isAttachedToForeignServer() const
@@ -1006,10 +997,8 @@ QCString DCOPClient::appId() const
 
 int DCOPClient::socket() const
 {
-#ifdef Q_OS_UNIX
     if (d->iceConn)
         return IceConnectionNumber(d->iceConn);
-#endif //Q_OS_UNIX
     return 0;
 }
 
@@ -1056,7 +1045,6 @@ QCString DCOPClient::senderId() const
 bool DCOPClient::send(const QCString &remApp, const QCString &remObjId,
                       const QCString &remFun, const QByteArray &data)
 {
-#ifdef Q_OS_UNIX
     if (remApp.isEmpty())
        return false;
     DCOPClient *localClient = findLocalClient( remApp );
@@ -1105,7 +1093,6 @@ bool DCOPClient::send(const QCString &remApp, const QCString &remObjId,
 
     if (IceConnectionStatus(d->iceConn) == IceConnectAccepted)
         return true;
-#endif //Q_OS_UNIX
     return false;
 }
 
@@ -1828,14 +1815,10 @@ bool DCOPClient::callInternal(const QCString &remApp, const QCString &remObjId,
                       QCString& replyType, QByteArray &replyData,
                       bool useEventLoop, int timeout, int minor_opcode)
 {
-#ifdef Q_OS_UNIX
     ReplyStruct replyStruct;
     replyStruct.replyType = &replyType;
     replyStruct.replyData = &replyData;
     return callInternal(remApp, remObjId, remFun, data, &replyStruct, useEventLoop, timeout, minor_opcode);
-#else //!Q_OS_UNIX
-    return false;
-#endif
 }
 
 bool DCOPClient::callInternal(const QCString &remApp, const QCString &remObjId,
@@ -1843,7 +1826,6 @@ bool DCOPClient::callInternal(const QCString &remApp, const QCString &remObjId,
                       ReplyStruct *replyStruct,
                       bool useEventLoop, int timeout, int minor_opcode)
 {
-#ifdef Q_OS_UNIX
     if ( !isAttached() )
         return false;
 
@@ -1992,9 +1974,6 @@ bool DCOPClient::callInternal(const QCString &remApp, const QCString &remObjId,
 
     d->currentKey = oldCurrentKey;
     return replyStruct->status != ReplyStruct::Failed;
-#else //!Q_OS_UNIX
-    return false;
-#endif
 }
 
 void DCOPClient::eventLoopTimeout()
@@ -2004,7 +1983,6 @@ void DCOPClient::eventLoopTimeout()
 
 void DCOPClient::processSocketData(int fd)
 {
-#ifdef Q_OS_UNIX
     // Make sure there is data to read!
     fd_set fds;
     timeval timeout;
@@ -2035,7 +2013,6 @@ void DCOPClient::processSocketData(int fd)
         qWarning("received an error processing data from the DCOP server!");
         return;
     }
-#endif //Q_OS_UNIX
 }
 
 void DCOPClient::setDefaultObject( const QCString& objId )
@@ -2077,9 +2054,7 @@ DCOPClient::beginTransaction()
     trans->id = ++d->transactionId;
     if (d->transactionId < 0)  // Ensure that ids > 0
         d->transactionId = 0;
-#ifdef Q_OS_UNIX
     trans->key = d->currentKey;
-#endif
 
     d->transactionList->append( trans );
 
@@ -2129,7 +2104,6 @@ DCOPClient::endTransaction( DCOPClientTransaction *trans, QCString& replyType,
         return;
     }
 
-#ifdef Q_OS_UNIX
     DCOPMsg *pMsg;
 
     QByteArray ba;
@@ -2142,7 +2116,6 @@ DCOPClient::endTransaction( DCOPClientTransaction *trans, QCString& replyType,
     pMsg->length += ba.size();
 
     IceSendData( d->iceConn, ba.size(), const_cast<char *>(ba.data()) );
-#endif
 
     delete trans;
 }
@@ -2232,7 +2205,6 @@ DCOPClient::disconnectDCOPSignal( const QCString &sender, const QCString &signal
 void
 DCOPClient::setPriorityCall(bool b)
 {
-#ifdef Q_OS_UNIX
     if (b)
     {
        if (d->currentKey == 2)
@@ -2248,7 +2220,6 @@ DCOPClient::setPriorityCall(bool b)
        if ( !d->messages.isEmpty() )
           d->postMessageTimer.start( 0, true ); // Process queued messages
     }
-#endif
 }
 
 
@@ -2265,7 +2236,6 @@ DCOPClient::emergencyClose()
        list.append(it.current());
        ++it;
     }
-#ifdef Q_OS_UNIX
     for(DCOPClient *cl = list.first(); cl; cl = list.next())
     {
         if (cl->d->iceConn) {
@@ -2274,7 +2244,6 @@ DCOPClient::emergencyClose()
             cl->d->iceConn = 0L;
         }
     }
-#endif
 }
 
 const char *
