@@ -1,5 +1,5 @@
 /*
-    $Id: kwinmodule.cpp,v 1.66 2004/11/29 00:18:01 aseigo Exp $
+    $Id: kwinmodule.cpp 590563 2006-09-30 13:33:05Z mueller $
 
     This file is part of the KDE libraries
     Copyright (C) 1999 Matthias Ettrich (ettrich@kde.org)
@@ -17,8 +17,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <qwidget.h>
@@ -36,21 +36,36 @@
 
 static KWinModulePrivate* static_d = 0;
 
-class KWinModulePrivate : public QWidget, public NETRootInfo
+static unsigned long windows_properties[ 2 ] = { NET::ClientList | NET::ClientListStacking |
+				     NET::NumberOfDesktops |
+				     NET::DesktopGeometry |
+                                     NET::DesktopViewport |
+				     NET::CurrentDesktop |
+				     NET::DesktopNames |
+				     NET::ActiveWindow |
+				     NET::WorkArea |
+				     NET::KDESystemTrayWindows,
+                                     NET::WM2ShowingDesktop };
+
+static unsigned long desktop_properties[ 2 ] = { 
+				     NET::NumberOfDesktops |
+				     NET::DesktopGeometry |
+                                     NET::DesktopViewport |
+				     NET::CurrentDesktop |
+				     NET::DesktopNames |
+				     NET::ActiveWindow |
+				     NET::WorkArea |
+				     NET::KDESystemTrayWindows,
+                                     NET::WM2ShowingDesktop };
+
+class KWinModulePrivate : public QWidget, public NETRootInfo4
 {
 public:
     KWinModulePrivate(int _what)
-	: QWidget(0,0), NETRootInfo( qt_xdisplay(),
-				     ( _what >= KWinModule::INFO_WINDOWS ? 
-				       (ClientList | ClientListStacking) : 0
-				     ) |
-				     NumberOfDesktops |
-				     DesktopGeometry |
-				     CurrentDesktop |
-				     DesktopNames |
-				     ActiveWindow |
-				     WorkArea |
-				     KDESystemTrayWindows,
+	: QWidget(0,0), NETRootInfo4( qt_xdisplay(),
+                                     _what >= KWinModule::INFO_WINDOWS ?
+                                     windows_properties : desktop_properties,
+                                     2,
 				     -1, false
 				     ),
           strutSignalConnected( false ),
@@ -93,8 +108,10 @@ public:
 
     void updateStackingOrder();
     bool removeStrutWindow( WId );
-};
 
+    QSize numberOfViewports(int desktop) const;
+    QPoint currentViewport(int desktop) const;
+};
 
 KWinModule::KWinModule( QObject* parent )
     : QObject( parent, "kwin_module" )
@@ -163,34 +180,65 @@ const QValueList<WId>& KWinModule::systemTrayWindows() const
     return d->systemTrayWindows;
 }
 
+QSize KWinModulePrivate::numberOfViewports(int desktop) const
+{
+    NETSize netdesktop = desktopGeometry(desktop);
+
+    return QSize(netdesktop.width / QApplication::desktop()->width(),
+            netdesktop.height / QApplication::desktop()->height());
+}
+
+QPoint KWinModulePrivate::currentViewport(int desktop) const
+{
+    NETPoint netviewport = desktopViewport(desktop);
+
+    return QPoint(1+(netviewport.x / QApplication::desktop()->width()),
+            1+(netviewport.y / QApplication::desktop()->height()));
+}
+
 bool KWinModulePrivate::x11Event( XEvent * ev )
 {
     if ( ev->xany.window == qt_xrootwin() ) {
         int old_current_desktop = currentDesktop();
         WId old_active_window = activeWindow();
         int old_number_of_desktops = numberOfDesktops();
-	int m = NETRootInfo::event( ev );
+        bool old_showing_desktop = showingDesktop();
+        unsigned long m[ 5 ];
+	NETRootInfo::event( ev, m, 5 );
 
-	if (( m & CurrentDesktop ) && currentDesktop() != old_current_desktop )
+	if (( m[ PROTOCOLS ] & CurrentDesktop ) && currentDesktop() != old_current_desktop )
 	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
 		emit (*mit)->currentDesktopChanged( currentDesktop() );
-	if (( m & ActiveWindow ) && activeWindow() != old_active_window )
+	if (( m[ PROTOCOLS ] & ActiveWindow ) && activeWindow() != old_active_window )
 	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
 		emit (*mit)->activeWindowChanged( activeWindow() );
-	if ( m & DesktopNames )
+	if ( m[ PROTOCOLS ] & DesktopViewport ) {
+	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
+		emit (*mit)->currentDesktopViewportChanged(currentDesktop(),
+                        currentViewport(currentDesktop()));
+        }
+	if ( m[ PROTOCOLS ] & DesktopGeometry ) {
+	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
+		emit (*mit)->desktopGeometryChanged(currentDesktop());
+	}
+	if ( m[ PROTOCOLS ] & DesktopNames )
 	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
 		emit (*mit)->desktopNamesChanged();
-	if (( m & NumberOfDesktops ) && numberOfDesktops() != old_number_of_desktops )
+	if (( m[ PROTOCOLS ] & NumberOfDesktops ) && numberOfDesktops() != old_number_of_desktops )
 	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
 		emit (*mit)->numberOfDesktopsChanged( numberOfDesktops() );
-	if ( m & WorkArea )
+	if ( m[ PROTOCOLS ] & WorkArea )
 	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
 		emit (*mit)->workAreaChanged();
-	if ( m & ClientListStacking ) {
+	if ( m[ PROTOCOLS ] & ClientListStacking ) {
 	    updateStackingOrder();
 	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
 		emit (*mit)->stackingOrderChanged();
 	}
+        if(( m[ PROTOCOLS2 ] & WM2ShowingDesktop ) && showingDesktop() != old_showing_desktop ) {
+	    for ( QPtrListIterator<KWinModule> mit( modules ); mit.current(); ++mit )
+		emit (*mit)->showingDesktopChanged( showingDesktop());
+        }
     } else  if ( windows.findIndex( ev->xany.window ) != -1 ){
 	NETWinInfo ni( qt_xdisplay(), ev->xany.window, qt_xrootwin(), 0 );
         unsigned long dirty[ 2 ];
@@ -205,7 +253,7 @@ bool KWinModulePrivate::x11Event( XEvent * ev )
         }
 	if ( (dirty[ NETWinInfo::PROTOCOLS ] & NET::WMStrut) != 0 ) {
             removeStrutWindow( ev->xany.window );
-            if ( !possibleStrutWindows.findIndex( ev->xany.window ) != -1  )
+            if ( possibleStrutWindows.findIndex( ev->xany.window ) == -1 )
         	possibleStrutWindows.append( ev->xany.window );
 	}
 	if ( dirty[ NETWinInfo::PROTOCOLS ] || dirty[ NETWinInfo::PROTOCOLS2 ] ) {
@@ -306,11 +354,26 @@ int KWinModule::numberOfDesktops() const
     return d->numberOfDesktops();
 }
 
+QSize KWinModule::numberOfViewports(int desktop) const
+{
+    return d->numberOfViewports(desktop);
+}
+
+QPoint KWinModule::currentViewport(int desktop) const
+{
+    return d->currentViewport(desktop);
+}
+
 WId KWinModule::activeWindow() const
 {
     return d->activeWindow();
 }
 
+bool KWinModule::showingDesktop() const
+{
+    return d->showingDesktop();
+}
+
 QRect KWinModule::workArea( int desktop ) const
 {
     int desk  = (desktop > 0 && desktop <= (int) d->numberOfDesktops() ) ? desktop : currentDesktop();
