@@ -609,7 +609,7 @@ bool KResolver::resolveAsync(QObject* userObj, const char *userSlot,
 
 QStrList KResolver::protocolName(int protonum)
 {
-  struct protoent *pe;
+  struct protoent *pe = 0L;
 #ifndef HAVE_GETPROTOBYNAME_R
   QMutexLocker locker(&getXXbyYYmutex);
 
@@ -628,6 +628,7 @@ QStrList KResolver::protocolName(int protonum)
       if (getprotobynumber_r(protonum, &protobuf, buf, buflen, &pe) == ERANGE)
 # endif
 	{
+          pe = 0L;
 	  buflen += 1024;
 	  delete [] buf;
 	}
@@ -655,7 +656,7 @@ QStrList KResolver::protocolName(int protonum)
 
 QStrList KResolver::protocolName(const char *protoname)
 {
-  struct protoent *pe;
+  struct protoent *pe = 0L;
 #ifndef HAVE_GETPROTOBYNAME_R
   QMutexLocker locker(&getXXbyYYmutex);
 
@@ -674,6 +675,7 @@ QStrList KResolver::protocolName(const char *protoname)
       if (getprotobyname_r(protoname, &protobuf, buf, buflen, &pe) == ERANGE)
 # endif
 	{
+          pe = 0L;
 	  buflen += 1024;
 	  delete [] buf;
 	}
@@ -701,7 +703,7 @@ QStrList KResolver::protocolName(const char *protoname)
 
 int KResolver::protocolNumber(const char *protoname)
 {
-  struct protoent *pe;
+  struct protoent *pe = 0L;
 #ifndef HAVE_GETPROTOBYNAME_R
   QMutexLocker locker(&getXXbyYYmutex);
 
@@ -720,6 +722,7 @@ int KResolver::protocolNumber(const char *protoname)
       if (getprotobyname_r(protoname, &protobuf, buf, buflen, &pe) == ERANGE)
 # endif
 	{
+          pe = 0L;
 	  buflen += 1024;
 	  delete [] buf;
 	}
@@ -743,7 +746,7 @@ int KResolver::protocolNumber(const char *protoname)
 
 int KResolver::servicePort(const char *servname, const char *protoname)
 {
-  struct servent *se;
+  struct servent *se = 0L;
 #ifndef HAVE_GETSERVBYNAME_R
   QMutexLocker locker(&getXXbyYYmutex);
 
@@ -762,6 +765,7 @@ int KResolver::servicePort(const char *servname, const char *protoname)
       if (getservbyname_r(servname, protoname, &servbuf, buf, buflen, &se) == ERANGE)
 # endif
 	{
+          se = 0L;
 	  buflen += 1024;
 	  delete [] buf;
 	}
@@ -785,7 +789,7 @@ int KResolver::servicePort(const char *servname, const char *protoname)
 
 QStrList KResolver::serviceName(const char* servname, const char *protoname)
 {
-  struct servent *se;
+  struct servent *se = 0L;
 #ifndef HAVE_GETSERVBYNAME_R
   QMutexLocker locker(&getXXbyYYmutex);
 
@@ -804,6 +808,7 @@ QStrList KResolver::serviceName(const char* servname, const char *protoname)
       if (getservbyname_r(servname, protoname, &servbuf, buf, buflen, &se) == ERANGE)
 # endif
 	{
+          se = 0L;
 	  buflen += 1024;
 	  delete [] buf;
 	}
@@ -831,7 +836,7 @@ QStrList KResolver::serviceName(const char* servname, const char *protoname)
 
 QStrList KResolver::serviceName(int port, const char *protoname)
 {
-  struct servent *se;
+  struct servent *se = 0L;
 #ifndef HAVE_GETSERVBYPORT_R
   QMutexLocker locker(&getXXbyYYmutex);
 
@@ -850,6 +855,7 @@ QStrList KResolver::serviceName(int port, const char *protoname)
       if (getservbyport_r(port, protoname, &servbuf, buf, buflen, &se) == ERANGE)
 # endif
 	{
+          se = 0L;
 	  buflen += 1024;
 	  delete [] buf;
 	}
@@ -936,7 +942,7 @@ static QStringList *KResolver_initIdnDomains()
 {
   const char *kde_use_idn = getenv("KDE_USE_IDN");
   if (!kde_use_idn)
-     kde_use_idn = "ac:at:br:ch:cl:cn:de:dk:fi:hu:info:io:jp:kr:li:lt:museum:no:se:sh:th:tm:tw:vn";
+     kde_use_idn = "ac:at:br:cat:ch:cl:cn:de:dk:fi:gr:hu:info:io:is:jp:kr:li:lt:museum:org:no:se:sh:th:tm:tw:vn";
   return new QStringList(QStringList::split(':', QString::fromLatin1(kde_use_idn).lower()));
 }
 
