@@ -23,8 +23,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef _KLINEEDIT_H
@@ -32,7 +32,6 @@
 
 #include <qlineedit.h>
 #include <kcompletion.h>
-#include <kdemacros.h>
 
 class QPopupMenu;
 
@@ -69,27 +68,27 @@ class KURL;
  * KLineEdit will by default handle the text rotation and completion
  * events internally when a completion object is created through either one
  * of the methods mentioned above.  If you do not need this functionality,
- * simply use KCompletionBase::setHandleSignals( bool ) or set the
- * boolean parameter in the above functions to false.
+ * simply use @p setHandleSignals( bool ) or set the boolean parameter in
+ * the above functions to false.
  *
  * The default key-bindings for completion and rotation is determined
- * from the global settings in KStdAccel.  These values, however,
- * can be overridden locally by invoking KCompletionBase::setKeyBinding().
- * The values can easily be reverted back to the default setting, by simply
- * calling useGlobalSettings(). An alternate method would be to default
+ * from the global settings in KStdAccel. These values, however, can
+ * be overridden locally by invoking @p setKeyBinding(). You can easily
+ * revert these settings back to the default by simply calling
+ * @p useGlobalSettings(). An alternate method would be to default
  * individual key-bindings by using setKeyBinding() with the default
  * second argument.
  *
- * If @p EchoMode for this widget is set to something other than @p QLineEdit::Normal, 
- * the completion mode will always be defaulted to KGlobalSettings::CompletionNone.  
+ * If @p EchoMode for this widget is set to something other than @p QLineEdit::Normal,
+ * the completion mode will always be defaulted to KGlobalSettings::CompletionNone.
  * This is done purposefully to guard against protected entries such as passwords being
  * cached in KCompletion's list. Hence, if the @p EchoMode is not QLineEdit::Normal, the
  * completion mode is automatically disabled.
  *
- * A read-only KLineEdit will have the same background color as a 
- * disabled KLineEdit, but its foreground color will be the one used 
+ * A read-only KLineEdit will have the same background color as a
+ * disabled KLineEdit, but its foreground color will be the one used
  * for the read-write mode. This differs from QLineEdit's implementation
- * and is done to give visual distinction between the three different modes: 
+ * and is done to give visual distinction between the three different modes:
  * disabled, read-only, and read-write.
  *
  * \b Usage \n
@@ -100,7 +99,7 @@ class KURL;
  * KLineEdit *edit = new KLineEdit( this, "mywidget" );
  * KCompletion *comp = edit->completionObject();
  * // Connect to the return pressed signal - optional
- * connect(edit,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&));
+ * connect(edit,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&)));
  * \endcode
  *
  * To use a customized completion objects or your
@@ -111,13 +110,13 @@ class KURL;
  * KURLCompletion *comp = new KURLCompletion();
  * edit->setCompletionObject( comp );
  * // Connect to the return pressed signal - optional
- * connect(edit,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&));
+ * connect(edit,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&)));
  * \endcode
  *
- * Note if you specify your own completion object you have to either delete 
- * it when you don't need it anymore, or you can tell KLineEdit to delete it 
+ * Note if you specify your own completion object you have to either delete
+ * it when you don't need it anymore, or you can tell KLineEdit to delete it
  * for you:
- * \code 
+ * \code
  * edit->setAutoDeleteCompletionObject( true );
  * \endcode
  *
@@ -143,7 +142,7 @@ class KURL;
  * @author Dawit Alemayehu <adawit@kde.org>
  */
 
-class KLineEdit : public QLineEdit, public KCompletionBase
+class KDEUI_EXPORT KLineEdit : public QLineEdit, public KCompletionBase
 {
     friend class KComboBox;
 
@@ -152,6 +151,8 @@ class KLineEdit : public QLineEdit, public KCompletionBase
     Q_PROPERTY( bool urlDropsEnabled READ isURLDropsEnabled WRITE setURLDropsEnabled )
     Q_PROPERTY( bool trapEnterKeyEvent READ trapReturnKey WRITE setTrapReturnKey )
     Q_PROPERTY( bool enableSqueezedText READ isSqueezedTextEnabled WRITE setEnableSqueezedText )
+    // @since 3.5.4
+    Q_PROPERTY( QString clickMessage READ clickMessage WRITE setClickMessage )
 
 public:
 
@@ -290,14 +291,15 @@ public:
      * Enable text squeezing whenever the supplied text is too long.
      * Only works for "read-only" mode.
      *
-     * Note that once text squeezing is enabled, @ref QLineEdit::text
-     * and @ref QLineEdit::displayText return the squeezed text. If
+     * Note that once text squeezing is enabled, QLineEdit::text()
+     * and QLineEdit::displayText() return the squeezed text. If
      * you want the original text, use @ref originalText.
      *
+     * @see QLineEdit
      * @since 3.2
-     */    
+     */
     void setEnableSqueezedText( bool enable );
-    
+
     /**
      * Returns true if text squeezing is enabled.
      * This is only valid when the widget is in read-only mode.
@@ -305,16 +307,41 @@ public:
      * @since 3.2
      */
     bool isSqueezedTextEnabled() const;
-    
+
     /**
-     * Returns the original text if text squeezing is enabled 
+     * Returns the original text if text squeezing is enabled.
      * If the widget is not in "read-only" mode, this function
-     * returns the same thing as @ref QLineEdit::text().
-     * 
+     * returns the same thing as QLineEdit::text().
+     *
+     * @see QLineEdit
      * @since 3.2
      */
     QString originalText() const;
 
+    /**
+     * Set the completion-box to be used in completion mode
+     * KGlobalSettings::CompletionPopup.
+     * This will do nothing if a completion-box already exists.
+     *
+     * @param box The KCompletionBox to set
+     * @since 3.4
+    */
+    void setCompletionBox( KCompletionBox *box );
+
+    /**
+     * This makes the line edit display a grayed-out hinting text as long as
+     * the user didn't enter any text. It is often used as indication about
+     * the purpose of the line edit.
+     * @since 3.5.4
+    */
+    void setClickMessage( const QString &msg );
+
+    /**
+     * @return the message set with setClickMessage
+     * @since 3.5.4
+    */
+    QString clickMessage() const;
+
 signals:
 
     /**
@@ -352,10 +379,11 @@ signals:
      * The argument indicates which key-binding was pressed.
      * In KLineEdit's case this can be either one of two values:
      * PrevCompletionMatch or NextCompletionMatch. See
-     * KCompletionBase::setKeyBinding for details.
+     * @p setKeyBinding for details.
      *
      * Note that this signal is @em not emitted if the completion
-     * mode is set to @p KGlobalSettings::CompletionNone or @p echoMode() is @em not  normal.
+     * mode is set to @p KGlobalSettings::CompletionNone or
+     * @p echoMode() is @em not  normal.
      */
     void textRotation( KCompletionBase::KeyBindingType );
 
@@ -405,12 +433,12 @@ public slots:
 
     /**
      * Sets @p items into the completion-box if completionMode() is
-     * CompletionPopup. The popup will be shown immediately. 
+     * CompletionPopup. The popup will be shown immediately.
      *
      * @param items list of completion matches to be shown in the completion box.
      */
     void setCompletedItems( const QStringList& items );
-    
+
     /**
      * Same as the above function except it allows you to temporarily
      * turn off text completion in CompletionPopupAuto mode.
@@ -440,8 +468,8 @@ public slots:
      * Re-implemented to enable text squeezing. API is not affected.
      */
     virtual void setText ( const QString& );
-    
-    
+
+
 protected slots:
 
     /**
@@ -459,10 +487,10 @@ protected slots:
      * @deprecated.  Will be removed in the next major release!
      */
     void slotCancelled() {}
-    
+
     /**
-     * Resets the current displayed text. 
-     * Call this function to revert a text completion if the user 
+     * Resets the current displayed text.
+     * Call this function to revert a text completion if the user
      * cancels the request. Mostly applies to popup completions.
      */
     void userCancelled(const QString & cancelText);
@@ -511,6 +539,13 @@ protected:
     */
     virtual QPopupMenu *createPopupMenu();
 
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QFrame::drawContents().
+    */
+    virtual void drawContents( QPainter *p );
+
     /**
     * Re-implemented to handle URI drops.
     *
@@ -541,6 +576,26 @@ protected:
     virtual void create( WId = 0, bool initializeWindow = true,
                          bool destroyOldWindow = true );
 
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QLineEdit::focusInEvent().
+    */
+    virtual void focusInEvent( QFocusEvent* );
+
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QLineEdit::focusOutEvent().
+    */
+    virtual void focusOutEvent( QFocusEvent* );
+
+    /**
+     * Whether in current state text should be auto-suggested
+     * @since 3.4
+    */
+    bool autoSuggest() const;
+
 private slots:
     void completionMenuActivated( int id );
     void tripleClickTimeout();  // resets possibleTripleClick
@@ -566,11 +621,6 @@ private:
      */
     void init();
 
-    /**
-     * Creates the completion box
-     */
-    void makeCompletionBox();
-
     /**
      * Checks whether we should/should not consume a key used as
      * an accelerator.
@@ -578,7 +628,7 @@ private:
     bool overrideAccel (const QKeyEvent* e);
 
     /**
-     * Properly sets the squeezed text whenever the widget is 
+     * Properly sets the squeezed text whenever the widget is
      * created or resized.
      */
     void setSqueezedText ();
