@@ -106,6 +106,8 @@ namespace KJS {
     FrameArray(ExecState *exec, KHTMLPart *p)
       : ObjectImp(exec->interpreter()->builtinObjectPrototype()), part(p) { }
     virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual Value call(ExecState *exec, Object &thisObj, const List &args);
+    virtual bool implementsCall() const { return true; }
   private:
     QGuardedPtr<KHTMLPart> part;
   };
@@ -128,7 +130,7 @@ namespace KJS {
 #include "rendering/render_replaced.h"
 
 ////////////////////// Screen Object ////////////////////////
-
+namespace KJS {
 // table for screen object
 /*
 @begin ScreenTable 7
@@ -218,7 +220,7 @@ Value Screen::getValueProperty(ExecState *exec, int token) const
 const ClassInfo Window::info = { "Window", &DOMAbstractView::info, &WindowTable, 0 };
 
 /*
-@begin WindowTable 89
+@begin WindowTable 162
   atob		Window::AToB		DontDelete|Function 1
   btoa		Window::BToA		DontDelete|Function 1
   closed	Window::Closed		DontDelete|ReadOnly
@@ -331,6 +333,69 @@ const ClassInfo Window::info = { "Window", &DOMAbstractView::info, &WindowTable,
   XMLHttpRequest Window::XMLHttpRequest DontDelete|ReadOnly
   XMLSerializer	Window::XMLSerializer	DontDelete|ReadOnly
   DOMParser	Window::DOMParser	DontDelete|ReadOnly
+
+# Mozilla dom emulation ones.
+  Element   Window::ElementCtor DontDelete
+  Document  Window::DocumentCtor DontDelete
+  #this one is an alias since we don't have a separate XMLDocument
+  XMLDocument Window::DocumentCtor DontDelete
+  HTMLElement  Window::HTMLElementCtor DontDelete
+  HTMLDocument  Window::HTMLDocumentCtor DontDelete
+  HTMLHtmlElement Window::HTMLHtmlElementCtor DontDelete
+  HTMLHeadElement Window::HTMLHeadElementCtor DontDelete
+  HTMLLinkElement Window::HTMLLinkElementCtor DontDelete
+  HTMLTitleElement Window::HTMLTitleElementCtor DontDelete
+  HTMLMetaElement Window::HTMLMetaElementCtor DontDelete
+  HTMLBaseElement Window::HTMLBaseElementCtor DontDelete
+  HTMLIsIndexElement Window::HTMLIsIndexElementCtor DontDelete
+  HTMLStyleElement Window::HTMLStyleElementCtor DontDelete
+  HTMLBodyElement Window::HTMLBodyElementCtor DontDelete
+  HTMLFormElement Window::HTMLFormElementCtor DontDelete
+  HTMLSelectElement Window::HTMLSelectElementCtor DontDelete
+  HTMLOptGroupElement Window::HTMLOptGroupElementCtor DontDelete
+  HTMLOptionElement Window::HTMLOptionElementCtor DontDelete
+  HTMLInputElement Window::HTMLInputElementCtor DontDelete
+  HTMLTextAreaElement Window::HTMLTextAreaElementCtor DontDelete
+  HTMLButtonElement Window::HTMLButtonElementCtor DontDelete
+  HTMLLabelElement Window::HTMLLabelElementCtor DontDelete
+  HTMLFieldSetElement Window::HTMLFieldSetElementCtor DontDelete
+  HTMLLegendElement Window::HTMLLegendElementCtor DontDelete
+  HTMLUListElement Window::HTMLUListElementCtor DontDelete
+  HTMLOListElement Window::HTMLOListElementCtor DontDelete
+  HTMLDListElement Window::HTMLDListElementCtor DontDelete
+  HTMLDirectoryElement Window::HTMLDirectoryElementCtor DontDelete
+  HTMLMenuElement Window::HTMLMenuElementCtor DontDelete
+  HTMLLIElement Window::HTMLLIElementCtor DontDelete
+  HTMLDivElement Window::HTMLDivElementCtor DontDelete
+  HTMLParagraphElement Window::HTMLParagraphElementCtor DontDelete
+  HTMLHeadingElement Window::HTMLHeadingElementCtor DontDelete
+  HTMLBlockQuoteElement Window::HTMLBlockQuoteElementCtor DontDelete
+  HTMLQuoteElement Window::HTMLQuoteElementCtor DontDelete
+  HTMLPreElement Window::HTMLPreElementCtor DontDelete
+  HTMLBRElement Window::HTMLBRElementCtor DontDelete
+  HTMLBaseFontElement Window::HTMLBaseFontElementCtor DontDelete
+  HTMLFontElement Window::HTMLFontElementCtor DontDelete
+  HTMLHRElement Window::HTMLHRElementCtor DontDelete
+  HTMLModElement Window::HTMLModElementCtor DontDelete
+  HTMLAnchorElement Window::HTMLAnchorElementCtor DontDelete
+  HTMLImageElement Window::HTMLImageElementCtor DontDelete
+  HTMLObjectElement Window::HTMLObjectElementCtor DontDelete
+  HTMLParamElement Window::HTMLParamElementCtor DontDelete
+  HTMLAppletElement Window::HTMLAppletElementCtor DontDelete
+  HTMLMapElement Window::HTMLMapElementCtor DontDelete
+  HTMLAreaElement Window::HTMLAreaElementCtor DontDelete
+  HTMLScriptElement Window::HTMLScriptElementCtor DontDelete
+  HTMLTableElement Window::HTMLTableElementCtor DontDelete
+  HTMLTableCaptionElement Window::HTMLTableCaptionElementCtor DontDelete
+  HTMLTableColElement Window::HTMLTableColElementCtor DontDelete
+  HTMLTableSectionElement Window::HTMLTableSectionElementCtor DontDelete
+  HTMLTableRowElement Window::HTMLTableRowElementCtor DontDelete
+  HTMLTableCellElement Window::HTMLTableCellElementCtor DontDelete
+  HTMLFrameSetElement Window::HTMLFrameSetElementCtor DontDelete
+  HTMLLayerElement Window::HTMLLayerElementCtor DontDelete
+  HTMLFrameElement Window::HTMLFrameElementCtor DontDelete
+  HTMLIFrameElement Window::HTMLIFrameElementCtor DontDelete
+  CSSStyleDeclaration Window::CSSStyleDeclarationCtor DontDelete
 @end
 */
 IMPLEMENT_PROTOFUNC_DOM(WindowFunc)
@@ -589,7 +654,7 @@ Value Window::get(ExecState *exec, const Identifier &p) const
       else
         return Undefined();
     case Node:
-      return getNodeConstructor(exec);
+      return NodeConstructor::self(exec);
     case Range:
       return getRangeConstructor(exec);
     case NodeFilter:
@@ -598,8 +663,126 @@ Value Window::get(ExecState *exec, const Identifier &p) const
       return getDOMExceptionConstructor(exec);
     case CSSRule:
       return getCSSRuleConstructor(exec);
+    case ElementCtor:
+      return ElementPseudoCtor::self(exec);
+    case HTMLElementCtor:
+      return HTMLElementPseudoCtor::self(exec);
+    case HTMLHtmlElementCtor:
+      return HTMLHtmlElementPseudoCtor::self(exec);
+    case HTMLHeadElementCtor:
+      return HTMLHeadElementPseudoCtor::self(exec);
+    case HTMLLinkElementCtor:
+      return HTMLLinkElementPseudoCtor::self(exec);
+    case HTMLTitleElementCtor:
+      return HTMLTitleElementPseudoCtor::self(exec);
+    case HTMLMetaElementCtor:
+      return HTMLMetaElementPseudoCtor::self(exec);
+    case HTMLBaseElementCtor:
+      return HTMLBaseElementPseudoCtor::self(exec);
+    case HTMLIsIndexElementCtor:
+      return HTMLIsIndexElementPseudoCtor::self(exec);
+    case HTMLStyleElementCtor:
+      return HTMLStyleElementPseudoCtor::self(exec);
+    case HTMLBodyElementCtor:
+      return HTMLBodyElementPseudoCtor::self(exec);
+    case HTMLFormElementCtor:
+      return HTMLFormElementPseudoCtor::self(exec);
+    case HTMLSelectElementCtor:
+      return HTMLSelectElementPseudoCtor::self(exec);
+    case HTMLOptGroupElementCtor:
+      return HTMLOptGroupElementPseudoCtor::self(exec);
+    case HTMLOptionElementCtor:
+      return HTMLOptionElementPseudoCtor::self(exec);
+    case HTMLInputElementCtor:
+      return HTMLInputElementPseudoCtor::self(exec);
+    case HTMLTextAreaElementCtor:
+      return HTMLTextAreaElementPseudoCtor::self(exec);
+    case HTMLButtonElementCtor:
+      return HTMLButtonElementPseudoCtor::self(exec);
+    case HTMLLabelElementCtor:
+      return HTMLLabelElementPseudoCtor::self(exec);
+    case HTMLFieldSetElementCtor:
+      return HTMLFieldSetElementPseudoCtor::self(exec);
+    case HTMLLegendElementCtor:
+      return HTMLLegendElementPseudoCtor::self(exec);
+    case HTMLUListElementCtor:
+      return HTMLUListElementPseudoCtor::self(exec);
+    case HTMLOListElementCtor:
+      return HTMLOListElementPseudoCtor::self(exec);
+    case HTMLDListElementCtor:
+      return HTMLDListElementPseudoCtor::self(exec);
+    case HTMLDirectoryElementCtor:
+      return HTMLDirectoryElementPseudoCtor::self(exec);
+    case HTMLMenuElementCtor:
+      return HTMLMenuElementPseudoCtor::self(exec);
+    case HTMLLIElementCtor:
+      return HTMLLIElementPseudoCtor::self(exec);
+    case HTMLDivElementCtor:
+      return HTMLDivElementPseudoCtor::self(exec);
+    case HTMLParagraphElementCtor:
+      return HTMLParagraphElementPseudoCtor::self(exec);
+    case HTMLHeadingElementCtor:
+      return HTMLHeadingElementPseudoCtor::self(exec);
+    case HTMLBlockQuoteElementCtor:
+      return HTMLBlockQuoteElementPseudoCtor::self(exec);
+    case HTMLQuoteElementCtor:
+      return HTMLQuoteElementPseudoCtor::self(exec);
+    case HTMLPreElementCtor:
+      return HTMLPreElementPseudoCtor::self(exec);
+    case HTMLBRElementCtor:
+      return HTMLBRElementPseudoCtor::self(exec);
+    case HTMLBaseFontElementCtor:
+      return HTMLBaseFontElementPseudoCtor::self(exec);
+    case HTMLFontElementCtor:
+      return HTMLFontElementPseudoCtor::self(exec);
+    case HTMLHRElementCtor:
+      return HTMLHRElementPseudoCtor::self(exec);
+    case HTMLModElementCtor:
+      return HTMLModElementPseudoCtor::self(exec);
+    case HTMLAnchorElementCtor:
+      return HTMLAnchorElementPseudoCtor::self(exec);
+    case HTMLImageElementCtor:
+      return HTMLImageElementPseudoCtor::self(exec);
+    case HTMLObjectElementCtor:
+      return HTMLObjectElementPseudoCtor::self(exec);
+    case HTMLParamElementCtor:
+      return HTMLParamElementPseudoCtor::self(exec);
+    case HTMLAppletElementCtor:
+      return HTMLAppletElementPseudoCtor::self(exec);
+    case HTMLMapElementCtor:
+      return HTMLMapElementPseudoCtor::self(exec);
+    case HTMLAreaElementCtor:
+      return HTMLAreaElementPseudoCtor::self(exec);
+    case HTMLScriptElementCtor:
+      return HTMLScriptElementPseudoCtor::self(exec);
+    case HTMLTableElementCtor:
+      return HTMLTableElementPseudoCtor::self(exec);
+    case HTMLTableCaptionElementCtor:
+      return HTMLTableCaptionElementPseudoCtor::self(exec);
+    case HTMLTableColElementCtor:
+      return HTMLTableColElementPseudoCtor::self(exec);
+    case HTMLTableSectionElementCtor:
+      return HTMLTableSectionElementPseudoCtor::self(exec);
+    case HTMLTableRowElementCtor:
+      return HTMLTableRowElementPseudoCtor::self(exec);
+    case HTMLTableCellElementCtor:
+      return HTMLTableCellElementPseudoCtor::self(exec);
+    case HTMLFrameSetElementCtor:
+      return HTMLFrameSetElementPseudoCtor::self(exec);
+    case HTMLLayerElementCtor:
+      return HTMLLayerElementPseudoCtor::self(exec);
+    case HTMLFrameElementCtor:
+      return HTMLFrameElementPseudoCtor::self(exec);
+    case HTMLIFrameElementCtor:
+      return HTMLIFrameElementPseudoCtor::self(exec);
+    case DocumentCtor:
+      return DocumentPseudoCtor::self(exec);
+    case HTMLDocumentCtor:
+      return HTMLDocumentPseudoCtor::self(exec);
+    case CSSStyleDeclarationCtor:
+        return CSSStyleDeclarationPseudoCtor::self(exec);
     case EventCtor:
-      return getEventConstructor(exec);
+      return EventConstructor::self(exec);
     case MutationEventCtor:
       return getMutationEventConstructor(exec);
     case KeyboardEventCtor:
@@ -1334,7 +1517,9 @@ Value Window::openWindow(ExecState *exec, const List& args)
     return Undefined();
   KHTMLView *widget = part->view();
   Value v = args[0];
-  QString str = v.toString(exec).qstring();
+  QString str;
+  if (v.isValid() && !v.isA(UndefinedType))
+    str = v.toString(exec).qstring();
 
   // prepare arguments
   KURL url;
@@ -1555,8 +1740,11 @@ Value WindowFunc::tryCall(ExecState *exec, Object &thisObj, const List &args)
 
   KHTMLView *widget = part->view();
   Value v = args[0];
-  UString s = v.toString(exec);
-  str = s.qstring();
+  UString s;
+  if (v.isValid() && !v.isA(UndefinedType)) {
+    s = v.toString(exec);
+    str = s.qstring();
+  }
 
   QString caption;
   if (part && !part->url().host().isEmpty())
@@ -1571,7 +1759,7 @@ Value WindowFunc::tryCall(ExecState *exec, Object &thisObj, const List &args)
       part->xmlDocImpl()->updateRendering();
     if ( part )
       emit part->browserExtension()->requestFocus(part);
-    KMessageBox::error(widget, QStyleSheet::convertFromPlainText(str), caption);
+    KMessageBox::error(widget, QStyleSheet::convertFromPlainText(str, QStyleSheetItem::WhiteSpaceNormal), caption);
     return Undefined();
   case Window::Confirm:
     if (!widget->dialogsAllowed())
@@ -2212,25 +2400,42 @@ Value FrameArray::get(ExecState *exec, const Identifier &p) const
   // hence, it can find non-frame things (and even let them hide frame ones!)
   // We don't quite do that, but do this as a fallback.
   DOM::DocumentImpl* doc  = static_cast<DOM::DocumentImpl*>(part->document().handle());
-  DOM::HTMLCollectionImpl docuAll(doc, DOM::HTMLCollectionImpl::DOC_ALL);
-  DOM::NodeImpl*     node = docuAll.namedItem(p.string());
-  if (node) {
-    if (node->id() == ID_FRAME || node->id() == ID_IFRAME) {
-      //Return the Window object.
-      KHTMLPart* part = static_cast<DOM::HTMLFrameElementImpl*>(node)->contentPart();
-      if (part)
-        return Value(Window::retrieveWindow(part));
-      else
-        return Undefined();
-    } else {
-      //Just a regular node..
-      return getDOMNode(exec, node);
+  if (doc) {
+    DOM::HTMLCollectionImpl docuAll(doc, DOM::HTMLCollectionImpl::DOC_ALL);
+    DOM::NodeImpl*     node = docuAll.namedItem(p.string());
+    if (node) {
+      if (node->id() == ID_FRAME || node->id() == ID_IFRAME) {
+        //Return the Window object.
+        KHTMLPart* part = static_cast<DOM::HTMLFrameElementImpl*>(node)->contentPart();
+        if (part)
+          return Value(Window::retrieveWindow(part));
+        else
+          return Undefined();
+      } else {
+        //Just a regular node..
+        return getDOMNode(exec, node);
+      }
     }
+  } else {
+    kdWarning(6070) << "Missing own document in FrameArray::get()" << endl;
   }
 
   return ObjectImp::get(exec, p);
 }
 
+Value FrameArray::call(ExecState *exec, Object &/*thisObj*/, const List &args)
+{
+    //IE supports a subset of the get functionality as call...
+    //... basically, when the return is a window, it supports that, otherwise it 
+    //errors out. We do a cheap-and-easy emulation of that, and just do the same
+    //thing as get does.
+    if (args.size() == 1)
+        return get(exec, Identifier(args[0].toString(exec)));
+
+    return Undefined();
+}
+
+
 ////////////////////// Location Object ////////////////////////
 
 const ClassInfo Location::info = { "Location", 0, &LocationTable, 0 };
@@ -2444,8 +2649,10 @@ Value LocationFunc::tryCall(ExecState *exec, Object &thisObj, const List &args)
     break;
   case Location::Reload: {
     KHTMLPart *khtmlpart = ::qt_cast<KHTMLPart *>(part);
-    if (part)
+    if (khtmlpart)
       khtmlpart->scheduleRedirection(-1, part->url().url(), true/*lock history*/);
+    else
+      part->openURL(part->url());
     break;
   }
   case Location::ToString:
@@ -2690,5 +2897,6 @@ UString Konqueror::toString(ExecState *) const
 
 #endif
 /////////////////////////////////////////////////////////////////////////////
+} //namespace KJS
 
 #include "kjs_window.moc"
