@@ -7,7 +7,8 @@
               (C) 1999 Lars Knoll (knoll@kde.org)
               (C) 1999 Antti Koivisto (koivisto@kde.org)
               (C) 2001-2003 Dirk Mueller (mueller@kde.org)
-              (C) 2002 Apple Computer, Inc.
+              (C) 2004 Apple Computer, Inc.
+              (C) 2006 Germain Garand (germain@ebooksfrance.org)
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -90,41 +91,39 @@ static const char titleEnd [] = "</title";
 #define fixUpChar(x) \
             switch ((x).unicode()) \
             { \
-            /* ALL of these should be changed to Unicode SOON */ \
             case 0x80: (x) = 0x20ac; break; \
-            case 0x82: (x) = ',';    break; \
+            case 0x82: (x) = 0x201a;    break; \
             case 0x83: (x) = 0x0192; break; \
-            case 0x84: (x) = '"';    break; \
+            case 0x84: (x) = 0x201e;    break; \
             case 0x85: (x) = 0x2026; break; \
             case 0x86: (x) = 0x2020; break; \
             case 0x87: (x) = 0x2021; break; \
             case 0x88: (x) = 0x02C6; break; \
             case 0x89: (x) = 0x2030; break; \
             case 0x8A: (x) = 0x0160; break; \
-            case 0x8b: (x) = '<';    break; \
+            case 0x8b: (x) = 0x2039;    break; \
             case 0x8C: (x) = 0x0152; break; \
             case 0x8E: (x) = 0x017D; break; \
-            case 0x91: (x) = '\'';   break; \
-            case 0x92: (x) = '\'';   break; \
-            case 0x93: (x) = '"';    break; \
-            case 0x94: (x) = '"';    break; \
-            case 0x95: (x) = '*';    break; \
-            case 0x96: (x) = '-';    break; \
-            case 0x97: (x) = '-';    break; \
-            case 0x98: (x) = '~';    break; \
+            case 0x91: (x) = 0x2018;   break; \
+            case 0x92: (x) = 0x2019;   break; \
+            case 0x93: (x) = 0x201C;    break; \
+            case 0x94: (x) = 0X201D;    break; \
+            case 0x95: (x) = 0x2022;    break; \
+            case 0x96: (x) = 0x2013;    break; \
+            case 0x97: (x) = 0x2014;    break; \
+            case 0x98: (x) = 0x02DC;    break; \
             case 0x99: (x) = 0x2122; break; \
             case 0x9A: (x) = 0x0161; break; \
-            case 0x9b: (x) = '>';    break; \
+            case 0x9b: (x) = 0x203A;    break; \
             case 0x9C: (x) = 0x0153; break; \
             case 0x9E: (x) = 0x017E; break; \
             case 0x9F: (x) = 0x0178; break; \
             default: break; \
             }
 #endif
-
 // ----------------------------------------------------------------------------
 
-HTMLTokenizer::HTMLTokenizer(DOM::DocumentPtr *_doc, KHTMLView *_view)
+HTMLTokenizer::HTMLTokenizer(DOM::DocumentImpl *_doc, KHTMLView *_view)
 {
     view = _view;
     buffer = 0;
@@ -139,7 +138,7 @@ HTMLTokenizer::HTMLTokenizer(DOM::DocumentPtr *_doc, KHTMLView *_view)
     reset();
 }
 
-HTMLTokenizer::HTMLTokenizer(DOM::DocumentPtr *_doc, DOM::DocumentFragmentImpl *i)
+HTMLTokenizer::HTMLTokenizer(DOM::DocumentImpl *_doc, DOM::DocumentFragmentImpl *i)
 {
     view = 0;
     buffer = 0;
@@ -339,7 +338,7 @@ void HTMLTokenizer::parseSpecial(TokenizerString &src)
                 else if ( title ) { currToken.tid = ID_TITLE + ID_CLOSE_TAG; }
                 else if ( xmp )  { currToken.tid = ID_XMP + ID_CLOSE_TAG; }
                 processToken();
-                style = script = style = textarea = title = xmp = false;
+                script = style = textarea = title = xmp = false;
                 tquote = NoQuote;
                 scriptCodeSize = scriptCodeResync = 0;
             }
@@ -387,42 +386,54 @@ void HTMLTokenizer::scriptHandler()
     currToken.tid = ID_SCRIPT + ID_CLOSE_TAG;
     processToken();
 
-    TokenizerString prependingSrc;
+    // Scripts following a frameset element should not be executed or even loaded in the case of extern scripts.
+    bool followingFrameset = (parser->doc()->body() && parser->doc()->body()->id() == ID_FRAMESET);
+    bool effectiveScript = !parser->skipMode() && !followingFrameset;
+    bool deferredScript = false;
 
-    if ( !parser->skipMode() ) {
+    if ( effectiveScript ) {
         CachedScript* cs = 0;
 
         // forget what we just got, load from src url instead
-        if ( !currentScriptSrc.isEmpty() &&
-             (cs = parser->doc()->docLoader()->requestScript(currentScriptSrc, scriptSrcCharset) ))
+        if ( !currentScriptSrc.isEmpty() && javascript &&
+             (cs = parser->doc()->docLoader()->requestScript(currentScriptSrc, scriptSrcCharset) )) {
             cachedScript.enqueue(cs);
+        }
 
         if (cs) {
-            pendingSrc.prepend(src);
+            pendingQueue.push(src);
+            uint scriptCount = cachedScript.count();
             setSrc(TokenizerString());
             scriptCodeSize = scriptCodeResync = 0;
             cs->ref(this);
+            if (cachedScript.count() == scriptCount)
+                deferredScript = true;
         }
         else if (currentScriptSrc.isEmpty() && view && javascript ) {
-            if ( !m_executingScript )
-                pendingSrc.prepend(src);
-            else
-                prependingSrc = src;
+            pendingQueue.push(src);
             setSrc(TokenizerString());
             scriptCodeSize = scriptCodeResync = 0;
             scriptExecution( exScript, QString::null, tagStartLineno /*scriptStartLineno*/ );
+        } else {
+            // script was filtered or disallowed
+            effectiveScript = false;
         }
     }
 
     script = false;
     scriptCodeSize = scriptCodeResync = 0;
 
+    if ( !effectiveScript )
+        return;
+
     if ( !m_executingScript && cachedScript.isEmpty() ) {
-        // kdDebug( 6036 ) << "adding pending Output to parsed string" << endl;
-        src.append(pendingSrc);
-        pendingSrc.clear();
-    } else if ( !prependingSrc.isEmpty() )
-        write( prependingSrc, false );
+        src.append(pendingQueue.pop());
+    } else if ( cachedScript.isEmpty() ) {
+        write( pendingQueue.pop(), false );
+    } else if ( !deferredScript && pendingQueue.count() > 1) {
+        TokenizerString t = pendingQueue.pop();
+        pendingQueue.top().prepend( t );
+    }
 }
 
 void HTMLTokenizer::scriptExecution( const QString& str, const QString& scriptURL,
@@ -446,7 +457,7 @@ void HTMLTokenizer::scriptExecution( const QString& str, const QString& scriptUR
 void HTMLTokenizer::parseComment(TokenizerString &src)
 {
     // SGML strict
-    bool strict = parser->doc()->inStrictMode() && parser->doc()->htmlMode() != DocumentImpl::XHtml;
+    bool strict = parser->doc()->inStrictMode() && parser->doc()->htmlMode() != DocumentImpl::XHtml && !script && !style;
     int delimiterCount = 0;
     bool canClose = false;
 
@@ -455,8 +466,7 @@ void HTMLTokenizer::parseComment(TokenizerString &src)
         scriptCode[ scriptCodeSize++ ] = *src;
 
 #if defined(TOKEN_DEBUG) && TOKEN_DEBUG > 1
-        qDebug("comment is now: *%s*",
-               QConstString((QChar*)src.current(), kMin(16U, src.length())).string().latin1());
+        qDebug("comment is now: *%s*", src.toString().left(16).latin1());
 #endif
 
         if (strict)
@@ -591,6 +601,7 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
     if( start )
     {
         cBufferPos = 0;
+        entityLen = 0;
         Entity = SearchEntity;
     }
 
@@ -686,8 +697,8 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
                 if ( tag == NoTag ) {
                     const entity* e = kde_findEntity(cBuffer, cBufferPos);
                     if ( e && e->code < 256 ) {
-                        Entity = SearchSemicolon;
-                        break;
+                        EntityChar = e->code;
+                        entityLen = cBufferPos;
                     }
                 }
             }
@@ -695,8 +706,12 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
             if(Entity == SearchSemicolon) {
                 if(cBufferPos > 1) {
                     const entity *e = kde_findEntity(cBuffer, cBufferPos);
-                    if(e && ( e->code < 256 || *src == ';' ))
+                    // IE only accepts unterminated entities < 256,
+                    // Gecko accepts them all, but only outside tags
+                    if(e && ( tag == NoTag || e->code < 256 || *src == ';' )) {
                         EntityChar = e->code;
+                        entityLen = cBufferPos;
+                    }
                 }
             }
             break;
@@ -712,7 +727,10 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
 
             if ( !EntityChar.isNull() ) {
                 checkBuffer();
-                // Just insert it
+                if (entityLen > 0 && entityLen < cBufferPos) {
+                    int rem = cBufferPos - entityLen;
+                    src.prepend( TokenizerString(QString::fromAscii(cBuffer+entityLen, rem)) );
+                }
                 src.push( EntityChar );
             } else {
 #ifdef TOKEN_DEBUG
@@ -724,7 +742,6 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
                 for(unsigned int i = 0; i < cBufferPos; i++)
                     dest[i] = cBuffer[i];
                 dest += cBufferPos;
-                Entity = NoEntity;
                 if (pre)
                     prePos += cBufferPos+1;
             }
@@ -746,10 +763,10 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
         checkBuffer();
 #if defined(TOKEN_DEBUG) && TOKEN_DEBUG > 1
         uint l = 0;
-        while(l < src.length() && (*(src.current()+l)).latin1() != '>')
+        while(l < src.length() && (src.toString()[l]).latin1() != '>')
             l++;
         qDebug("src is now: *%s*, tquote: %d",
-               QConstString((QChar*)src.current(), l).string().latin1(), tquote);
+               src.toString().left(l).latin1(), tquote);
 #endif
         switch(tag) {
         case NoTag:
@@ -775,10 +792,6 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         tag = NoTag;
 
                         comment = true;
-                        // push what we parsed so far upon the stack. helps for <!-->
-                        checkScriptBuffer();
-                        scriptCode[0] = scriptCode[1] = '-';
-                        scriptCodeSize = 2;
                         parseComment(src);
                         return; // Finished parsing tag!
                     }
@@ -898,8 +911,16 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         unsigned int a;
                         cBuffer[cBufferPos] = '\0';
                         a = khtml::getAttrID(cBuffer, cBufferPos);
-                        if ( !a )
-                            attrName = QString::fromLatin1(QCString(cBuffer, cBufferPos+1).data());
+                        if ( !a ) {
+                            // did we just get /> or e.g checked/>
+                            if (curchar == '>' && cBufferPos >=1 && cBuffer[cBufferPos-1] == '/') {
+                                currToken.flat = true;
+                                if (cBufferPos>1)
+                                    a = khtml::getAttrID(cBuffer, cBufferPos-1);
+                            }
+                            if (!a)
+                                attrName = QString::fromLatin1(QCString(cBuffer, cBufferPos+1).data());
+                        }
 
                         dest = buffer;
                         *dest++ = a;
@@ -909,9 +930,6 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         else
                             kdDebug( 6036 ) << "Known attribute: " << QCString(cBuffer, cBufferPos+1).data() << endl;
 #endif
-                        // did we just get />
-                        if (!a && cBufferPos == 1 && *cBuffer == '/' && curchar == '>')
-                            currToken.flat = true;
 
                         tag = SearchEqual;
                         break;
@@ -955,7 +973,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     }
                     else {
                         DOMString v("");
-                        currToken.addAttribute(parser->docPtr()->document(), buffer, attrName, v);
+                        currToken.addAttribute(parser->docPtr(), buffer, attrName, v);
                         dest = buffer;
                         tag = SearchAttribute;
                     }
@@ -1010,7 +1028,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         while(dest > buffer+1 && (*(dest-1) == '\n' || *(dest-1) == '\r'))
                             dest--; // remove trailing newlines
                         DOMString v(buffer+1, dest-buffer-1);
-                        currToken.addAttribute(parser->docPtr()->document(), buffer, attrName, v);
+                        currToken.addAttribute(parser->docPtr(), buffer, attrName, v);
 
                         dest = buffer;
                         tag = SearchAttribute;
@@ -1046,7 +1064,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     if ( curchar <= ' ' || curchar == '>' )
                     {
                         DOMString v(buffer+1, dest-buffer-1);
-                        currToken.addAttribute(parser->docPtr()->document(), buffer, attrName, v);
+                        currToken.addAttribute(parser->docPtr(), buffer, attrName, v);
                         dest = buffer;
                         tag = SearchAttribute;
                         break;
@@ -1142,6 +1160,8 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         type.compare("text/ecmascript") != 0 &&
                         type.compare("text/livescript") != 0 &&
 			type.compare("application/x-javascript") != 0 &&
+			type.compare("application/x-ecmascript") != 0 &&
+			type.compare("application/javascript") != 0 &&
 			type.compare("application/ecmascript") != 0 )
                         javascript = false;
                 } else if( a ) {
@@ -1299,10 +1319,14 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
     if ( !buffer )
         return;
 
-    if ( ( m_executingScript && appendData ) ||
-         ( !m_executingScript && cachedScript.count() ) ) {
+    if ( ( m_executingScript && appendData ) || cachedScript.count() ) {
         // don't parse; we will do this later
-        pendingSrc.append(str);
+        if (pendingQueue.isEmpty())
+            pendingQueue.push(str);
+        else if (appendData)
+            pendingQueue.bottom().append(str);
+        else
+            pendingQueue.top().append(str);
         return;
     }
 
@@ -1311,7 +1335,10 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
         return;
     }
 
-    setSrc(str);
+    if (!src.isEmpty())
+        src.append(str);
+    else
+        setSrc(str);
     m_abort = false;
 
 //     if (Entity)
@@ -1710,9 +1737,9 @@ void HTMLTokenizer::notifyFinished(CachedObject* /*finishedObj*/)
     assert(!cachedScript.isEmpty());
     bool done = false;
     while (!done && cachedScript.head()->isLoaded()) {
-#ifdef TOKEN_DEBUG
+
         kdDebug( 6036 ) << "Finished loading an external script" << endl;
-#endif
+
         CachedScript* cs = cachedScript.dequeue();
         DOMString scriptSource = cs->script();
 #ifdef TOKEN_DEBUG
@@ -1730,12 +1757,16 @@ void HTMLTokenizer::notifyFinished(CachedObject* /*finishedObj*/)
         done = cachedScript.isEmpty();
 
         // 'script' is true when we are called synchronously from
-        // parseScript(). In that case parseScript() will take care
+        // scriptHandler(). In that case scriptHandler() will take care
         // of 'scriptOutput'.
         if ( !script ) {
-            TokenizerString rest = pendingSrc;
-            pendingSrc.clear();
-            write(rest, false);
+            while (pendingQueue.count() > 1) {
+               TokenizerString t = pendingQueue.pop();
+               pendingQueue.top().prepend( t );
+            }
+            if (done) {
+                write(pendingQueue.pop(), false);
+            }
             // we might be deleted at this point, do not
             // access any members.
         }
