@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "kbookmarkmanager.h"
@@ -44,6 +44,8 @@
 
 class KBookmarkManagerPrivate : public dPtrTemplate<KBookmarkManager, KBookmarkManagerPrivate> {
 public:
+    KBookmarkManagerPrivate()
+        { m_browserEditor = true; }
     QString m_editorCaption;
     bool m_browserEditor;
 };
@@ -529,7 +531,8 @@ KBookmarkGroup KBookmarkManager::addBookmarkDialog(
     return parentBookmark;
 }
 
-void KBookmarkManager::emitChanged( KBookmarkGroup & group )
+
+void KBookmarkManager::emitChanged( /*KDE4 const*/ KBookmarkGroup & group )
 {
     save();
 
@@ -560,12 +563,8 @@ void KBookmarkManager::notifyCompleteChange( QString caller ) // DCOP call
     // Reparse
     parse();
     // Tell our GUI
-    // (emit with group == "" to directly mark the root menu as dirty)
+    // (emit where group is "" to directly mark the root menu as dirty)
     emit changed( "", caller );
-    // Also tell specifically about the toolbar - unless it's the root as well
-    KBookmarkGroup tbGroup = toolbar();
-    if (!tbGroup.isNull() && !tbGroup.groupAddress().isEmpty())
-        emit changed( tbGroup.groupAddress(), caller );
 }
 
 void KBookmarkManager::notifyConfigChanged() // DCOP call
@@ -598,7 +597,7 @@ bool KBookmarkManager::showNSBookmarks() const
 void KBookmarkManager::setShowNSBookmarks( bool show )
 {
     m_showNSBookmarks = show;
-    if (this != userBookmarksManager())
+    if (this->path() != userBookmarksFile())
        return;
     KBookmarkMenu::DynMenuInfo info
        = KBookmarkMenu::showDynamicBookmarks("netscape");
@@ -689,10 +688,14 @@ void KBookmarkManager::updateFavicon( const QString &url, const QString &favicon
     }
 }
 
+QString KBookmarkManager::userBookmarksFile()
+{
+    return locateLocal("data", QString::fromLatin1("konqueror/bookmarks.xml"));
+}
+
 KBookmarkManager* KBookmarkManager::userBookmarksManager()
 {
-   QString bookmarksFile = locateLocal("data", QString::fromLatin1("konqueror/bookmarks.xml"));
-   return KBookmarkManager::managerForFile( bookmarksFile );
+   return KBookmarkManager::managerForFile( userBookmarksFile() );
 }
 
 KBookmarkSettings* KBookmarkSettings::s_self = 0;
